/*  Autopackage GTK+ frontend
 *  Copyright (C) 2003  Hongli Lai <h.lai@chello.nl>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifndef _PREFS_C_
#define _PREFS_C_

#include <stdio.h>
#include <string.h>
#include "prefs.h"
#include "utils.h"


gchar *
pref_get (gchar *prefName)
{
	gchar *filename;
	FILE *f;
	gchar buf[1024];

	filename = g_strdup_printf ("%s/.autopackage-gtk",
		g_get_home_dir ());

	f = fopen (filename, "r");
	if (!f)
	{
		g_free (filename);
		return g_strdup ("");
	}

	while (!feof (f) && fgets (buf, sizeof (buf), f))
	{
		gchar *name, *value;

		stripNewline (buf);
		/* Ignore empty lines, comments and lines that don't contain a = */
		if (!*buf || *buf == '#' || !strchr (buf, '='))
			continue;

		value = strchr (buf, '=') + 1;
		name = g_strndup (buf, value - buf - 1);
		if (strcmp (prefName, name) == 0)
		{
			g_free (name);
			return g_strdup (value);
		}
		g_free (name);
	}
	fclose (f);
	return g_strdup ("");
}


gboolean
pref_get_bool (gchar *prefName)
{
	gchar *value;

	value = pref_get (prefName);
	if (g_strcasecmp (value, "true") == 0
	 || g_strcasecmp (value, "1") == 0
	 || g_strcasecmp (value, "yes") == 0)
	{
		g_free (value);
		return TRUE;
	} else
	{
		g_free (value);
		return FALSE;
	}
}


void
pref_set (gchar *prefName, gchar *value)
{
	FILE *f;
	gchar buf[1024];
	GList *lines = NULL, *list;
	gboolean append = TRUE;
	gchar *filename;

	filename = g_strdup_printf ("%s/.autopackage-gtk",
		g_get_home_dir ());

	f = fopen (filename, "r");
	if (!f)
		goto doWrite;

	while (!feof (f) && fgets (buf, sizeof (buf), f))
		lines = g_list_append (lines, g_strdup (buf));
	fclose (f);


	for (list = lines; list; list = list->next)
	{
		gchar *line = (gchar *) list->data;
		gchar *name;

		if (!*line || *line == '#' || !strchr (line, '='))
			continue;

		name = g_strndup (line, strchr (line, '=') - line);
		if (strcmp (name, prefName) == 0)
		{
			append = FALSE;
			g_free (list->data);
			list->data = g_strdup_printf ("%s=%s\n", prefName, value);
			g_free (name);
			break;
		}
		g_free (name);
	}

doWrite:
	if (append)
		lines = g_list_append (lines, g_strdup_printf ("%s=%s\n", prefName, value));


	f = fopen (filename, "w");
	if (!f)
	{
		g_free (filename);
		return;
	}
	for (list = lines; list; list = list->next)
	{
		gchar *line = (gchar *) list->data;
		fprintf (f, line);
	}
	fclose (f);
	free_strlist (lines);
	g_free (filename);
}


#endif /* _PREFS_C_ */
